################################################################################
#    R LABORATORY                                                              #
#                                                                              #
#    Lessons 24 October and 8 November 2019                                                      #
#                                                                              #
#    If you have any questions please contact me at:                           #
#    francesco.schirripa@ec.unipi.it                                           #
#                                                                              #
#    Room 238 2^nd floor, DEM                                                  #
#                                                                              #
################################################################################

rm(list=ls(all=TRUE))


# Estimation of Laeken indicators using synthetic EU-SILC data

# load the library laeken
library(laeken)


# The Laeken indicators is a set of common European statistical indicators 
# on poverty and social exclusion established at the European Council of 
# December 2001 in the Brussels suburb of Laeken, Belgium. 

# A more complete overview of the available functionality is given 
# in additional package vignettes on specialized topics. 
# A list of the available vignettes can be viewed from within R with the
# following command:
vignette(package="laeken")

# Load synthetic database of Austria EU-SILC 2006
data(eusilc)
dim(eusilc)


# Only a few of the large number of variables in the original survey 
# are included in the example dataset. 

# METADATA EU-SILC 2006
# https://www.gesis.org/en/missy/metadata/EU-SILC/2006/Cross-sectional/original



# equivalized household size according to the modified OECD scale 
# needs to be computed.

# This can be done with the function eqSS(), 
# which requires the household ID and the age of the
# individuals as arguments.  

# eqSS():
# Compute the equivalized household size according to the modified OECD scale 
# adopted in 1994.

# eqSS(hid, age, year = NULL, data = NULL)

# hid: household ID
# age: age of the individuals
# year: a vector containing the year of the survey
# data: a data.frame containing EU-SILC survey

# In the example data, household ID and age are stored in 
# the variables db030 and age, respectively
eusilc$eqSS <- eqSS("db030", "age", data=eusilc)
head(eusilc[,c("db030", "age", "eqSS")], 10)


# eqInc(hid, hplus, hminus, pplus, pminus, eqSS, year = NULL, data = NULL)
# hid: the  household  ID,
# hplus: the  household  income  components to be added
# hminus: the  household  income  components to be subtracted
# pplus: the personal income  components to be added
# pminus: the personal income  components to be subtracted
# eqSS: the equivalized household size

# The net income is considered in this example, therefore no personal income component
# needs to be subtracted

# hy040n: income from rental of a property or land (net). 
# hy050n: family/children related allowances (net). 
# hy070n: housing allowances (net).
# hy080n: regular inter-household cash transfer received (net).
# hy090n: interest, dividends, profit from capital investments in unincorporated business (net).
# hy110n: income received by people aged under 16 (net).

hplus <- c("hy040n", "hy050n", "hy070n", "hy080n", "hy090n", "hy110n")


# hy130n: regular inter-household cash transfer paid (net).
# hy145n: repayments/receipts for tax adjustment (net).

hminus <- c("hy130n", "hy145n")

# py010n numeric; employee cash or near cash income (net).
# py050n numeric; cash benefits or losses from self-employment (net). 
# py090n numeric; unemployment benefits (net).
# py100n numeric; old-age benefits (net).
# py110n numeric; survivor's benefits (net).
# py120n numeric; sickness benefits (net).
# py130n numeric; disability benefits (net).
# py140n numeric; education-related allowances (net).

pplus <- c("py010n", "py050n", "py090n", "py100n",
           "py110n", "py120n", "py130n", "py140n")


eusilc$eqIncome <- eqInc("db030", hplus, hminus,
                         pplus, pminus=NULL, "eqSS", data=eusilc)

head(eusilc[,c("db030", "eqSS", "eqIncome")], 10)


# ARPT
# poverty threshold: is equal to the 60% of the national median equivalised income
#help("weightedMedian")
poverty.line<-weightedMedian(eusilc$eqIncome, eusilc$rb050)*0.6
poverty.line

# arpt can be calculated also with function arpt()
arpt(eusilc$eqIncome, weights = eusilc$rb050)


# Define if an individual is poor assign 1 if his 
# eqIncome is below the at-risk-at-poverty line
risk.pov<- ifelse(eusilc$eqIncome <= poverty.line, 1, 0)
eusilc$risk.pov<-risk.pov

# calulcate HCR as weighted mean of people that have an 
# income below the poverty line 
HCR<-weighted.mean(risk.pov, w=eusilc$rb050)
HCR
HCR*100


# the function arpr() is implemented in laeken to estimate the HCR

# arpr(inc, weights = NULL, sort = NULL, years = NULL, breakdown = NULL,
#      design = NULL, cluster = NULL, data = NULL, p = 0.6, var = NULL,
#      alpha = 0.05, na.rm = FALSE, ...)

# the function gives also the povety threshold
arpr(eusilc$eqIncome, weights = eusilc$rb050)



# dispersion around the at-risk-of-poverty threshold,
# defined as the proportion of persons with an equivalized disposable 
# income below 40%, 50% and 70% of the national weighted median 
# equivalized disposable income. 
# For the estimation of this indicator with function arpr(), 
# the proportion of the median equivalized
# income to be used can easily be adjusted via the argument p.
arpr("eqIncome", weights = "rb050", p = c(0.4, 0.5, 0.7), data = eusilc)


# HCR for subdomains 

# federal states level (NUTS 2)

# NUTS: Nomenclature of Territorial Units for Statistics 
# it is a common statistical classification of territorial units in  order  to  allow  
# the  collection,  compilation and  dissemination  of  European  statistics  at  different  
# territorial  levels  of  the  European Union.


# In order to estimate the HCR at NUTS 2 (identified by variable db040)
# we have to calculate the weighted mean of risk.pov for each regions
# Explore the variable db040
str(eusilc$db040)
table (eusilc$db040)
#build a dataframe with name of regions and frequencies
tab_db040<-data.frame(table(eusilc$db040))
View(tab_db040)

# calculate the HCR:
# measures the proportion of the population that is poor, i.e. the proportion of 
# a population that lives below the poverty line
HCR_NUTS2<-as.numeric(lapply(split(eusilc, eusilc$db040), function(eusilc) weighted.mean(eusilc$risk.pov, eusilc$rb050)))
HCR_AUT2<-data.frame(REGION=tab_db040$Var1,HCR_NUTS2=HCR_NUTS2*100)
HCR_AUT2
# We can calcuate the HCR in each regions by using the function arpr() 
# setting the breakdown argument (categorical variable!)
arpr(eusilc$eqIncome, weights = eusilc$rb050, breakdown = eusilc$db040)


# Any kind of breakdown can be supplied                                                                            
# 2009). With the following lines of code, a breakdown variable with all possible combinations of age
ageCat <- cut(eusilc$age, c(-1, 16, 25, 50, 65, Inf), right=FALSE)
eusilc$breakdown <- paste(ageCat, eusilc$rb090, sep=":")
arpr("eqIncome", weights = "rb050", breakdown = "breakdown", data = eusilc)


# HCR with RELATIVE POVERTY LINE

# reative poverty line 
rel.povertyline<-as.numeric(lapply(split(eusilc, eusilc$db040), function(eusilc) weightedMedian(eusilc$eqIncome, eusilc$rb050)))*0.6
# everything is okay? Proof seeing if using one region I Obtain the same result!
# Burgenland <- eusilc[which(eusilc$db040 == "Burgenland"),]
# weightedMedian(Burgenland$eqIncome, Burgenland$rb050)*0.6

# add the variable at the dataframe for calculate the HCR on the basis of 
# the relative poverty line
# Firstly, order the dataset because the values in 
# rel.povertyline are ordered on the basis
# of db040 
# (I prefer to do not work on the original dataset but I define an other 
# dataframe)
eusilc2 <- eusilc[order(eusilc$db040),] 
# then repeat the relative poverty line for the number of obs in each region
eusilc2$rel.poveryline=rep(rel.povertyline, tab_db040$Freq)

# Define if an individual is poor assign 1 if his eqIncome is below the 
# relative poverty line
eusilc2$rel.risk.pov<- ifelse(eusilc2$eqIncome <= eusilc2$rel.poveryline, 1, 0)

#use the same function above to compute the HCR
HCR_rel<-as.numeric(lapply(split(eusilc2, eusilc2$db040), function(eusilc2) weighted.mean(eusilc2$rel.risk.pov, eusilc2$rb050)))
AUT_POV<-data.frame(REGION=tab_db040$Var1, 
                    ABS_POV.LINE=poverty.line,
                    HCR_abs=HCR_NUTS2*100,
                    REL_POV.LINE=rel.povertyline,
                    HCR_rel=HCR_rel*100
)
AUT_POV

# RELATIVE POVERTY GAP
# PG is  given  by  the  difference  between  the median equivalized 
# disposable income of persons below the at-risk-of-poverty threshold 
# and the  at-risk  of  poverty  threshold  itself  

# it measures the poverty intensity, since it takes into account 
# the distance from the poverty line.

# build a dataset with only poors
poors<-eusilc [ (eusilc$eqIncome<poverty.line),]
#calcuate the median
poors_med<-weightedMedian(poors$eqIncome, poors$rb050)
# calculate the RPG
# =0 implies that no one in the population is below the poverty line. 
# =100 implies that everyone in the population has zero income.
PG<-(poverty.line-poors_med)/poverty.line
PG*100
# The RPG is implemented in the function rmpg()
rmpg("eqIncome", weights = "rb050", data = eusilc)
rmpg("eqIncome", weights = "rb050", breakdown = "db040", data = eusilc)

# GINI
# For estimating the Gini coefficient, the function gini() can be used.
gini("eqIncome", weights = "rb050", data = eusilc)
gini("eqIncome", weights = "rb050", breakdown = "db040", data = eusilc)

# QUINTILE SHARE RATIO 
# To estimate the quintile share ratio, the function qsr() is available.
qsr("eqIncome", weights = "rb050", data = eusilc)
qsr("eqIncome", weights = "rb050", breakdown = "db040", data = eusilc)
# The weightedQuantile() command computes the weighed 20% quantile, 
# the weighted median, and the weighted 80% quantile. 
weightedQuantile(eusilc$eqIncome, eusilc$rb050,
                 probs = c(0.2, 0.5, 0.8))


# VARIANCE ESTIMATION

args(bootVar)

# inc: the income vector.
# weights: an optional vector of sample weights.
# breakdown: an optional vector giving different domains in which 
# variances and confidence intervals should be computed.
# design: an optional vector or factor giving different strata for 
# stratified sampling designs.
# indicator: an object inheriting from the class "indicator" that 
# contains the point estimates of the
# indicator, such as "arpr" for the at-risk-of-poverty rate, "qsr" for the quintile share ratio,
# "rmpg" for the relative median at-risk-of-poverty gap, or "gini" for the Gini coefficient.
# bootType a character string specifying the type of bootstap to be 
# performed. Possible values are "calibrate" 
# (for calibration of the sample weights of the resampled observations  
# in every iteration) and "naive" (for a naive bootstrap without 
# calibration of the sample weights).
# ciType a character string specifying the type of confidence intervals
# Possible values are "perc", "norm" and "basic"
# alpha significance level to be used for computing the confidence intervals

# computing confidence intervals: national level
hcr.aut <- arpr(eusilc$eqIncome, weights = eusilc$rb050) 
bootVar("eqIncome", weights = "rb050",
        data = eusilc, indicator = hcr.aut, R = 5, 
        bootType="naive", ciType="perc", seed = 8)
#design: an optional vector or factor giving different strata 
# for stratified sampling designs
# a  stratified  sampling  design  can  be considered by specifying the
# design argument, in which case observations are resampled separately 
# within the strata. 

bootVar("eqIncome", weights = "rb050", design = "db040",
         data = eusilc, indicator = hcr.aut, 
        bootType="naive", ciType="perc", seed = 8)

## bootstrap with calibration
bootVar("eqIncome", weights = "rb050", design = "db040",
        data = eusilc, indicator = hcr.aut,
        X = calibVars(eusilc$db040), seed = 8)

# Using the breakdown argument, the example below produces variance and 
# confidence interval estimates for each NUTS2 region in addition 
# to the overall values.
hcr.2 <- arpr("eqIncome", weights = "rb050", breakdown = "db040", data = eusilc)

var.2<-bootVar("eqIncome", weights = "rb050", breakdown = "db040", 
               design = "db040",
               data = eusilc, indicator = hcr.2, 
               bootType = "naive", seed = 8)

RESULTS<-data.frame(var.2$ciByStratum, hcr=hcr.2$valueByStratum[,2])

# plotting confidence interval

RESULTS <- RESULTS[order(RESULTS$hcr), ]
RESULTS <- cbind(RESULTS, c(1:dim(RESULTS)[1]))
colnames(RESULTS)[dim(RESULTS)[2]] <- "rank"

library(plotrix)
plotCI(RESULTS$rank, RESULTS$hcr, 
       ui=RESULTS$upper, 
       li=RESULTS$lower,
       xlab= "Districts (sorted by ARPR)",
       ylab = "ARPR", 
       main="Confidence Interval Plot", 
       ylim=c(9,30),xaxt="n", col="red")

text(1:9,1.2*RESULTS$hcr,RESULTS$stratum, 
     cex=0.6,srt = 70, adj=c(-0.1,0.3))


#--------------------------------------------------------------------------------------
#
#                                          MAPPING 
#
#--------------------------------------------------------------------------------------
# in the following we use R to plot in a map the HCR at NUTS2 level in Austria

library(maptools)
library(RColorBrewer)
library(classInt)
library(sp)
library(lattice)

# In order to plot a map we have to load the shape file corresponding to the Country of interest
# Shapefiles are so-called vector databases, describing the location of points 
# (e.g., collecting locations), polylines (e.g., roads)  andareas (or polygons, e.g., soil types,  
# countries). A shapefile actually consists of three separate files with the same name 
# but with different extensions (SHP, SHX and DBF), but they are treated as one file. 

# you can use this website to download shape files:
# http://www.diva-gis.org/datadown

# NSI provide shape files 

# use function readShapePoly() from package maptools
# The readShapePoly() reads data from a polygon shapefile into a SpatialPolygonsDataFrame object.
# inside the folder AUT_adm I have a series of file
# the file useful for me in this case is AUT_adm1.shp
Austria=readShapePoly("C:/Users/Utente/OneDrive - University of Pisa/Esercitazioni UniPi/Lezioni JM 2019/Lezioni I sem JM/Data/AUT_adm/AUT_adm1.shp")
# Warning meassage because the use of this function is deprecated and it is not being maintained
# but it works!

# Alterantively you can use rgdal::readOGR()
# install.packages("rgdal") #install package
library(rgdal)
# in this case I load Austria shapefile with NUTS-3 border cahnginf the name of file
# I use AUT_adm2.shp
Austria3<-rgdal::readOGR("C:/Users/Utente/OneDrive - University of Pisa/Esercitazioni UniPi/Lezioni JM 2019/Lezioni I sem JM/Data/AUT_adm/AUT_adm2.shp")


# plot to see the difference of Austria and Austria3
plot(Austria)
plot (Austria3) # more detailed maps

# The I have to merge my SpatialPolygonsDataFrame object (Austria) with my data
# To do this I need a common variable
View(Austria)
# Looking at the file AUT_adm1.dbf I can see that each Region is identified by means of a ID
# 1	Burgenland
# 2	Carinthia
# 3	Lower Austria
# 4	Upper Austria
# 5	Salzburg
# 6	Styria
# 7	Tyrol
# 8	Vorarlberg
# 9	Vienna

# So I add at my dataframe containing the HCR and the name of the regions an ID 
# for each region according to UT_adm1.dbf

HCR_AUT2$ID_1=c(1,2,3,5,6,7,4,9,8)
HCR_AUT2

AUT<- merge(Austria, HCR_AUT2, by="ID_1")
# help(spplot) (this command is in the library {sp})
# plotting data (intial maps with no custom options)
spplot(AUT, "HCR_NUTS2", main = "HCR NUTS 2")

# If you don't like the default color scheme, you can use your own with ColorBrewer, 
# which you can install with install.packages("RColorBrewer"). 
# Once loaded, you can see a list of all the color pallets that come with RColorBrewer 
# with the command: 
display.brewer.all()

# Once you've picked a palette you like, create a palette object as follows, 
# where n is the number of cuts you want to use, and name is the name of the color ramp. 
# Note that different palettes have different limits on the 
# maximium and minimium number of cuts  allowed.
pal1 <- brewer.pal(n=5,name="OrRd") 
# Then you can just pass this pallet to spplot, making sure to set the number of cuts 
# to 1 minus the number of colors.
# cuts: number of levels the range of 'z' would be divided into 
spplot(AUT, "HCR_NUTS2", main = "HCR NUTS 2", col.regions = pal1, cuts = 4)

# If you don't want equally spaced cutoffs between colors, you can use the classInt 
# library to make custom cuts.
nclr <- 4
plotclr <- brewer.pal(nclr,"OrRd")
#style	chosen style: one of "fixed", "sd", "equal", "pretty", 
#                   "quantile", "kmeans", "hclust", "bclust", "fisher", or "jenks"
# intervalClosure (default "left"), allows specification of whether 
# partition intervals are closed on the left or the right 
class <- classIntervals(AUT$HCR_NUTS2, nclr, style="quantile")
#breakpoints
categ = class[["brks"]]
categ
# quantiles of the HCR
quantile(AUT$HCR_NUTS2)

#assing col at each class
colcode <- findColours(class, plotclr)
#round the extremities of the class
lab=round(categ,3)
#plot (adding colorkey I can change the lateral bar with labels)
spplot(AUT, "HCR_NUTS2",
       col.regions=plotclr, 
       at=c(class$brks[1], class$brks[2],class$brks[3],class$brks[4], class$brks[5]+0.01),
       colorkey = list(space="bottom",height = 0.9,
                       labels=list(at = lab, labels = lab)))       

# You can also plot the HCR divide in categories according to quantiles
# define class and add the variable at the dataframe 
AUT$cutHCR <- cut(AUT$HCR_NUTS2, breaks = categ, 
                  right = F, include.lowest=T)  
#plot (in this case I set dimensions of the bar in the legend of equal lenght )
spplot(AUT, "cutHCR", col.regions=plotclr, 
       main = "HCR AUSTRIA (NUTS 2)",
       colorkey = list(space="bottom",height = 0.9, 
                       labels = list(at = seq(0.5, length(categ) -0.5), 
                                     labels = round(categ,2))))
